"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelMiddleware = exports.OmitUpdateInput = exports.OmitCreateInput = void 0;
const cm_libs_utils_1 = require("cm-libs-utils");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
var RecordStatus;
(function (RecordStatus) {
    RecordStatus["ACTIVE"] = "ACTIVE";
    RecordStatus["INACTIVE"] = "INACTIVE";
})(RecordStatus || (RecordStatus = {}));
exports.OmitCreateInput = { createdBy: 0, createdByName: '' };
exports.OmitUpdateInput = { updatedBy: 0, updatedByName: '' };
class ModelMiddleware {
    constructor(client, model, accessInfo, errorHandler, schemas) {
        if (!(0 <= accessInfo?.requestedBy)) {
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'RequestedBy parameter is missing', { accessInfo });
        }
        accessInfo?.requestedByName || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'RequestedByName parameter is missing', { accessInfo });
        client['$use'](async (params, next) => {
            const start = (0, cm_libs_utils_1.timeNow)();
            switch (params.action) {
                case 'create':
                case 'createMany':
                    params = this.packCreateArgs(params);
                    break;
                case 'update':
                case 'updateMany':
                    params = this.packUpdateArgs(params);
                    break;
                case 'upsert':
                    params = this.packUpsertArgs(params);
                    break;
            }
            try {
                const result = await next(params);
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.NORMAL, () => `Run ${params.model}.${params.action} affected ${affectedCount(params.action, result)} record(s) took ${(0, cm_libs_utils_1.timeElapsed)(start)} ms`);
                return result;
            }
            catch (error) {
                errorHandler ? errorHandler(error, params, accessInfo) : (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.DATABASE_ERROR, { params, accessInfo });
            }
        });
        this.client = client;
        this.model = client[model] || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.INVALID_PARAMETER, 'Model is not exists', { model });
        this.schemas = schemas;
        this.accessInfo = accessInfo;
    }
    packCreateArgs(params) {
        const schema = this.schemas?.schemaCreateInput;
        const requestedBy = this.accessInfo.requestedBy;
        const requestedByName = this.accessInfo.requestedByName;
        if (!Array.isArray(params.args.data)) {
            params.args.data = {
                ...validatedValues(params.args.data, schema),
                createdBy: requestedBy,
                createdByName: requestedByName
            };
            return params;
        }
        for (let i = 0; params.args.data.length > i; i++) {
            params.args.data[i] = {
                ...validatedValues(params.args.data[i], schema),
                createdBy: requestedBy,
                createdByName: requestedByName
            };
        }
        return params;
    }
    packUpdateArgs(params) {
        const schema = this.schemas?.schemaUpdateInput;
        const requestedBy = this.accessInfo.requestedBy;
        const requestedByName = this.accessInfo.requestedByName;
        if (!Array.isArray(params.args.data)) {
            params.args.data = {
                ...validatedValues(params.args.data, schema),
                updatedBy: requestedBy,
                updatedByName: requestedByName
            };
            return params;
        }
        for (let i = 0; params.args.data.length > i; i++) {
            params.args.data[i] = {
                ...validatedValues(params.args.data[i], schema),
                updatedBy: requestedBy,
                updatedByName: requestedByName
            };
        }
        return params;
    }
    packUpsertArgs(params) {
        const schemaCreateInput = this.schemas?.schemaCreateInput;
        const schemaUpdateInput = this.schemas?.schemaUpdateInput;
        const requestedBy = this.accessInfo.requestedBy;
        const requestedByName = this.accessInfo.requestedByName;
        params.args.create = {
            ...validatedValues(params.args.create, schemaCreateInput),
            createdBy: requestedBy,
            createdByName: requestedByName
        };
        params.args.update = {
            ...validatedValues(params.args.update, schemaUpdateInput),
            updatedBy: requestedBy,
            updatedByName: requestedByName
        };
        return params;
    }
    async browse(params) {
        params ??= {};
        const pagination = params?.pagination || {};
        const options = params?.options || {};
        delete params.pagination;
        delete params.options;
        pagination.page = !(0 < pagination.page) ? 1 : pagination.page;
        pagination.pageSize ??= 10;
        params.where ??= {
            status: RecordStatus.ACTIVE,
            ...params.where
        };
        params.orderBy ??= { createdAt: 'desc' };
        if (0 < pagination.pageSize) {
            params.take = pagination.pageSize;
            params.skip = (pagination.page - 1) * pagination.pageSize;
        }
        const promises = [this.model['findMany'](params)];
        if (!(options.speedMode ??= false)) {
            promises.push(this.model['count']({ where: params.where }));
        }
        const results = await Promise.all(promises);
        const ret = {
            items: results[0],
            properties: options.speedMode ? pagination : { ...pagination, count: results[1] }
        };
        return ret;
    }
}
exports.ModelMiddleware = ModelMiddleware;
function affectedCount(action, result) {
    let count;
    switch (action) {
        case 'findUnique':
        case 'update':
            count = result ? 1 : 0;
            break;
        case 'findMany':
            count = result?.length;
            break;
        case 'count':
            count = result;
            break;
        case 'create':
        case 'upsert':
            count = 1;
            break;
        case 'createMany':
        case 'updateMany':
            count = result?.count;
    }
    return count || 0;
}
function validatedValues(data, schema) {
    if (!schema) {
        return data;
    }
    const { error, value } = schema.validate(data, { stripUnknown: true });
    if (error) {
        (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.VALIDATION_ERROR, { data });
    }
    return value;
}
//# sourceMappingURL=model-middleware.js.map